@@ -1,7 +1,10 @@
 import datetime
 import logging
+import re
 from enum import Enum
 from db_hammer.csv import start as csv_start
+from db_hammer.entity_util import get_insert_sql, get_update_sql
+from db_hammer.sql_exception import SqlException
 from db_hammer.util.date import date_to_str
 
 
@@ -15,6 +18,7 @@ class BaseConnection(object):
         """
         :param kwargs:
         """
+        self.log = kwargs.get("log", logging.getLogger(__name__))
         self.Db_NAME = kwargs.get("db_name")
         self.conn = None  # type:Connect
         self.cursor = None  # type:cursor
@@ -26,24 +30,55 @@ class BaseConnection(object):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.close()
 
-    def select_page_size(self, sql: str, page_size=50) -> int:
+    def select_page_size(self, sql: str, params: dict = None, page_size=50) -> int:
         """
         根据sql获取页数
+        :param params:
         :param sql:
         :param page_size: 分页大小
         :return: 页数
         """
 
         count_sql = "select COUNT(0) from ( %s ) temp_count" % sql
-        logging.debug("execute sql ==>:" + count_sql.replace("\n", " "))
-        self.cursor.execute(count_sql)
+        count_sql, p_list = self.sql_params(count_sql, params)
+        self.log.debug("execute sql ==>:" + count_sql.replace("\n", " "))
+        self.cursor.execute(count_sql, p_list)
         data = self.cursor.fetchone()
-        logging.debug("fetch rows  <==:" + str(len(data)))
+        self.log.debug("fetch rows  <==:" + str(len(data)))
         num = data[0] // page_size
         if data[0] > page_size * num:
             num = num + 1
         return num
 
+    def sql_params(self, sql: str, params: dict):
+        if params is None:
+            return sql
+        p_list = []
+        ps = re.findall('(?<!\w):\w+', sql)
+        start = 0
+        for p in ps:
+            if p[1:] not in params or len(p) == 1:
+                raise SqlException(f"SQL语句中参数[{p}]没有传值")
+            value = params.get(p[1:])
+            if isinstance(value, (str, datetime.date, datetime.datetime)):
+                start = sql.find(p, start)
+                sql = sql[:start] + "%s" + sql[start + len(p):]
+                start += 2
+                p_list.append(value)
+            elif isinstance(value, int):
+                start = sql.find(p, start)
+                sql = sql[:start] + "%d" + sql[start + len(p):]
+                start += 2
+                p_list.append(value)
+            elif isinstance(value, float):
+                start = sql.find(p, start)
+                sql = sql[:start] + "%f" + sql[start + len(p):]
+                start += 2
+                p_list.append(value)
+            else:
+                raise SqlException(f"传值不支持类型：" + str(type(value)))
+        return sql, p_list
+
     def select_page_list(self, sql: str, page_size=50, page_start=1, **kwargs) -> list:
         """
         获取分页列表数据,列表方式返回
@@ -56,6 +91,7 @@ class BaseConnection(object):
         add_headers: 是否添加表头 defalut:False
         :return:
         """
+        params = kwargs.get("params", None)
         page_end = kwargs["page_end"]
         if page_end is None:
             page_end = page_start + 1
@@ -66,14 +102,14 @@ class BaseConnection(object):
         start = page_size * (page_start - 1)
         end = page_size * (page_end - 1)
         page_sql = """select * FROM (select temp_page.*, rownum row_id from 
-                     ( %s ) temp_page 
-                   where limit %d,%d
-                   """ % (sql, end, start)
-
-        logging.debug("execute sql ==>:" + page_sql.replace("\n", " "))
-        self.cursor.execute(page_sql)
+                         ( %s ) temp_page 
+                       where limit %d,%d
+                       """ % (sql, end, start)
+        page_sql, p_list = self.sql_params(page_sql, params)
+        self.log.debug("execute sql ==>:" + page_sql.replace("\n", " "))
+        self.cursor.execute(page_sql, p_list)
         data = self.cursor.fetchall()
-        logging.debug("fetch rows  <==:" + str(len(data)))
+        self.log.debug("fetch rows  <==:" + str(len(data)))
         if add_headers:
             col_names = self.cursor.description
             list(data).insert(col_names, 0)
@@ -89,62 +125,65 @@ class BaseConnection(object):
         :return:
         """
 
-        page_end = kwargs["page_end"]
-        if page_end is None:
-            page_end = page_start + 1
-        add_headers = kwargs["add_headers"]
-        if add_headers is None:
-            add_headers = False
-
-        data = self.select_page_list(sql, page_size, page_start, page_end=page_end, add_headers=add_headers)
+        page_end = kwargs.get("page_end", page_start + 1)
+        add_headers = kwargs.get("add_headers", False)
+        params = kwargs.get("params", None)
+        data = self.select_page_list(sql, page_size, page_start, params=params,
+                                     page_end=page_end,
+                                     add_headers=add_headers)
         col_names = self.cursor.description
         return self._data_to_map(col_names, data)
 
-    def select_value(self, sql: str) -> str:
+    def select_value(self, sql: str, params=None) -> str:
         """
         获取第一列第一行的值
         :param sql:
         :return:
         """
-        logging.debug("execute sql ==>:" + sql.replace("\n", " "))
-        self.cursor.execute(sql)
+        self.log.debug("execute sql ==>:" + sql.replace("\n", " "))
+        _sql, p_list = self.sql_params(sql, params)
+        self.cursor.execute(_sql, p_list)
         data = self.cursor.fetchone()
         if data is None:
             return None
         return str(data[0])
 
-    def select_list(self, sql: str) -> list:
+    def select_list(self, sql: str, params=None) -> list:
         """
         获取列表数据,列表方式返回
         :param sql:
         :return:
         """
-        logging.debug("execute sql ==>:" + sql.replace("\n", " "))
-        self.cursor.execute(sql)
+        self.log.debug("execute sql ==>:" + sql.replace("\n", " "))
+        _sql, p_list = self.sql_params(sql, params)
+        self.cursor.execute(_sql, p_list)
         data = self.cursor.fetchall()
-        logging.debug("fetch rows  <==:" + str(len(data)))
+        self.log.debug("fetch rows  <==:" + str(len(data)))
         return data
 
-    def select_dict_list(self, sql: str) -> list:
+    def select_dict_list(self, sql: str, params=None) -> list:
         """
         获取查询语句列表数据，以字典方式返回: List<dict>
+        :param params:
         :param sql:
         :return:
         """
-        logging.debug("execute sql ==>:" + sql.replace("\n", " "))
-        self.cursor.execute(sql)
+        self.log.debug("execute sql ==>:" + sql.replace("\n", " "))
+        _sql, p_list = self.sql_params(sql, params)
+        self.cursor.execute(_sql, p_list)
         data = self.cursor.fetchall()
-        logging.debug("fetch rows  <==:" + str(len(data)))
+        self.log.debug("fetch rows  <==:" + str(len(data)))
         col_names = self.cursor.description
         return self._data_to_map(col_names, data)
 
-    def select_dict(self, sql: str) -> dict:
+    def select_dict(self, sql: str, params=None) -> dict:
         """
         获取字典数据，以字典方式返回: dict
+        :param params:
         :param sql:
         :return:
         """
-        data = self.select_dict_list(sql)
+        data = self.select_dict_list(sql, params)
         if data is None or len(data) == 0:
             return None
         return data[0]
@@ -159,34 +198,35 @@ class BaseConnection(object):
             r_list.append(row)
         return r_list
 
-    def execute(self, sql: str) -> int:
+    def execute(self, sql: str, params=None) -> int:
         """
         执行sql,并返回影响行数
+        :param params:
         :param sql:
         :return:
         """
-        logging.debug("execute sql ==>:" + sql.replace("\n", " "))
-        self.cursor.execute(sql)
+        self.log.debug("execute sql ==>:" + sql.replace("\n", " "))
+        self.cursor.execute(sql, params)
         i = self.cursor.rowcount
-        logging.debug("fetch rows  <==:" + str(i))
+        self.log.debug("fetch rows  <==:" + str(i))
         return i
 
     def close(self):
         self.conn.close()
-        logging.debug("conn close")
+        self.log.debug("conn close")
 
     def rollback(self):
         """回滚事务"""
-        logging.debug("rollback start")
+        self.log.debug("rollback start")
         self.conn.rollback()
-        logging.debug("rollback end")
+        self.log.debug("rollback end")
 
     def commit(self):
         """提交事务，如果有对数据库进更新，需要手动提交事务
         """
-        logging.debug("commit start")
+        self.log.debug("commit start")
         self.conn.commit()
-        logging.debug("commit end")
+        self.log.debug("commit end")
         return
 
     def db_data_type_mapping(self):
@@ -209,7 +249,7 @@ class BaseConnection(object):
             dd = dt.split(",")
             if str(db_type).upper() in dd:
                 return dtm[dt]
-        raise Exception("不支持的数据库类：" + db_type)
+        raise Exception("不支持的数据库类：" + str(db_type))
 
     def db_column_sql(self, table_name):
         """
@@ -226,16 +266,16 @@ class BaseConnection(object):
             DATA_PRECISION ：精度
         """
         return f"""
-        SELECT 
-         COLUMN_NAME AS COLUMN_NAME,
-        DATA_TYPE AS DATA_TYPE,
-        COLUMN_COMMENT AS COMMENTS,
-        CHARACTER_MAXIMUM_LENGTH AS DATA_LENGTH,
-        NUMERIC_PRECISION AS DATA_PRECISION
-         FROM INFORMATION_SCHEMA.COLUMNS T
-         WHERE 
-         TABLE_NAME = '{table_name}' AND TABLE_SCHEMA = '{self.Db_NAME}'
-        """
+            SELECT 
+             COLUMN_NAME AS COLUMN_NAME,
+            DATA_TYPE AS DATA_TYPE,
+            COLUMN_COMMENT AS COMMENTS,
+            CHARACTER_MAXIMUM_LENGTH AS DATA_LENGTH,
+            NUMERIC_PRECISION AS DATA_PRECISION
+             FROM INFORMATION_SCHEMA.COLUMNS T
+             WHERE 
+             TABLE_NAME = '{table_name}' AND TABLE_SCHEMA = '{self.Db_NAME}'
+            """
 
     def gen_insert_dict_sql(self, dict_data: dict, table_name: str):
         """
@@ -250,7 +290,7 @@ class BaseConnection(object):
             for c in columns:
                 column_name = c.get("COLUMN_NAME", None) if c.get("COLUMN_NAME", None) is not None else c["column_name"]
                 data_type = c.get("DATA_TYPE", None) if c.get("DATA_TYPE", None) is not None else c["data_type"]
-                self.__get_column_values(column_name, data_type, dict_data, values)
+                values[column_name] = self.__get_column_value(data_type=data_type, value=dict_data[column_name])
         else:
             return None
         sql = f"""INSERT INTO {self.Db_NAME}.{table_name} ({','.join(values.keys())}) VALUES ({','.join(values.values())})"""
@@ -258,7 +298,7 @@ class BaseConnection(object):
 
     def __get_column(self, name, columns):
         for c in columns:
-            column_name = c.get("COLUMN_NAME", None) if c.get("COLUMN_NAME", None) is None else c["column_name"]
+            column_name = c.get("COLUMN_NAME", None) if c.get("COLUMN_NAME", None) is not None else c["column_name"]
             if column_name == name:
                 return c
         return None
@@ -272,7 +312,7 @@ class BaseConnection(object):
             self.table_column_cache[table_name] = columns
             return columns
 
-    def gen_update_dict_sql(self, dict_data: dict, table_name: str, where: str):
+    def gen_update_dict_sql(self, dict_data: dict, table_name: str, where: str, not_update_colunms=[]):
         """
         根据字典生成Update语句
         :param dict_data:  列新字典
@@ -286,31 +326,33 @@ class BaseConnection(object):
             for column_name in dict_data.keys():
                 column = self.__get_column(column_name, columns)
                 if column is not None:
-                    data_type = column.get("DATA_TYPE", None) if column.get("DATA_TYPE", None) is None else column[
+                    data_type = column.get("DATA_TYPE", None) if column.get("DATA_TYPE", None) is not None else column[
                         "data_type"]
-                    self.__get_column_values(column_name, data_type, dict_data, values)
+                    values[column_name] = self.__get_column_value(data_type=data_type, value=dict_data[column_name])
         else:
             return None
         sets = []
         for c in values.keys():
-            sets.append(f"{c}={values[c]}")
+            if c not in not_update_colunms:
+                sets.append(f"{c}={values[c]}")
         update_sql = f"UPDATE {self.Db_NAME}.{table_name} SET {','.join(sets)} " + where
         return update_sql
 
-    def __get_column_values(self, column_name, data_type, dict_data, values):
-        v = dict_data.get(column_name, None)
+    def __get_column_value(self, data_type, value):
+        if data_type is None:
+            return f"'{self.convert_str(value)}'"
         t = self.__get_py_type(db_type=data_type)
-        if v is None:
-            values[column_name] = "null"
+        if value is None:
+            return "null"
         elif t == str:
-            values[column_name] = f"'{self.convert_str(v)}'"
+            return f"'{self.convert_str(value)}'"
         elif t in (int, float):
-            values[column_name] = str(v)
+            return str(value)
         else:
-            if isinstance(v, datetime.datetime) or isinstance(v, datetime.date):
-                values[column_name] = t.replace("${VALUE}", date_to_str(v))
+            if isinstance(value, (datetime.datetime, datetime.date)):
+                return t.replace("${VALUE}", date_to_str(value))
             else:
-                values[column_name] = t.replace("${VALUE}", self.convert_str(v))
+                return t.replace("${VALUE}", self.convert_str(value))
 
     def select_insert_sql(self, sql: str, table_name: str) -> list:
         """根据SQL查询数据并生成Insert语句"""
@@ -349,4 +391,32 @@ class BaseConnection(object):
                   CSV_SPLIT=data_split_chars,
                   CSV_FIELD_CLOSE=data_close_chars,
                   encoding=encoding,
-                  callback=outingCallback)
+                  callback=outingCallback,
+                  log=self.log)
+
+    def insert_entity(self, entity):
+        """传入实体保存到数据库"""
+        sql = get_insert_sql(entity=entity, fn_db_value=self.__get_column_value)
+        self.log.debug(sql)
+        return self.execute(sql)
+
+    def update_entity(self, entity):
+        sql = get_update_sql(entity=entity, fn_db_value=self.__get_column_value)
+        self.log.debug(sql)
+        # return self.execute(sql)
+
+    def update_entity_select(self, entity):
+        sql = get_update_sql(entity=entity, fn_db_value=self.__get_column_value)
+        self.log.debug(sql)
+
+    def delete_entity_by_primary_key(self):
+        print("delete_by_primary_key")
+
+    def delete_entity_by_columns(self):
+        print("delete_by_columns")
+
+    def select_entity_list_by_columns(self):
+        print("select_list_by_columns")
+
+    def select_entity_one_by_columns(self):
+        print("select_one_by_columns")
